// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.alta.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.nationrel.modules.alta.constant.AltaConstants;
import com.nationrel.modules.alta.entity.AltaAnalysis;
import com.nationrel.modules.alta.entity.AltaAnalysisInfo;
import com.nationrel.modules.alta.general.common.ServerResponse;
import com.nationrel.modules.alta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.alta.general.domain.enums.common.AuthorityProgramExceptionEnum;
import com.nationrel.modules.alta.general.util.common.BasicDataUtil;
import com.nationrel.modules.alta.mapper.AltaAnalysisInfoMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.Objects;

/**
 * @ClassName AltaAnalysisInfoService
 * @Description 加速数据分析数据的操作
 * @Author hcs
 * @CreateDate 2021-06-26
 * @Version V1.0
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AltaAnalysisInfoService {

    private final AltaAnalysisInfoMapper analysisInfoMapper;
    private final AltaAnalysisService analysisService;

    /**
     * 分页查询
     *
     * @param userInfo
     * @param altaName
     * @param startDate
     * @param endDate
     * @param pageIndex
     * @param pageSize
     * @return
     */
    public Result<IPage<AltaAnalysisInfo>> listByPage(HeaderUserInfo userInfo, String altaId, String altaName, Date startDate, Date endDate, int pageIndex, int pageSize) {
        // 创建分页条件
        Page<AltaAnalysisInfo> page = new Page<>(pageIndex, pageSize);
        // 创建基础查询条件
        LambdaQueryWrapper<AltaAnalysisInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AltaAnalysisInfo::getCreateBy, userInfo.getUserName());
        queryWrapper.eq(AltaAnalysisInfo::getDelFlag, AltaConstants.DEFAULT_DEL_FLAG);
        if (StringUtils.isNotBlank(altaId)) {
            queryWrapper.like(AltaAnalysisInfo::getAltaId, altaId);
        }
        if (StringUtils.isNotBlank(altaName)) {
            queryWrapper.like(AltaAnalysisInfo::getAltaName, altaName);
        }
        if (startDate != null) {
            queryWrapper.ge(AltaAnalysisInfo::getCreateTime, startDate);
        }
        if (endDate != null) {
            queryWrapper.le(AltaAnalysisInfo::getCreateTime, endDate);
        }
        queryWrapper.orderByDesc(AltaAnalysisInfo::getUpdateTime);
        return Result.ok(analysisInfoMapper.selectPage(page, queryWrapper));
    }

    /**
     * 添加分析数据信息
     *
     * @param userInfo
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public Result<String> addAnalysis(HeaderUserInfo userInfo, AltaAnalysisInfo entity) {

        String altaId = BasicDataUtil.uuidGenerator();
        entity.setAltaId(altaId);
        entity.setCreateTime(new Date());
        entity.setUpdateTime(new Date());
        entity.setCreateBy(userInfo.getUserName());
        entity.setUpdateBy(userInfo.getUserName());
        entity.setDelFlag(AltaConstants.DEFAULT_DEL_FLAG);
        analysisInfoMapper.insert(entity);
        // 添加分析数据
        AltaAnalysis analysis = new AltaAnalysis();
        analysis.setAltaId(altaId);
        analysis.setAnalysisName("alta-a");
        analysisService.addAnalysis(userInfo, analysis);

        return Result.ok();
    }

    /**
     * 更新修改者信息
     * @param userInfo
     * @param altaId
     */
    public static void updateUpdateInfo(HeaderUserInfo userInfo, String altaId, AltaAnalysisInfoMapper analysisInfoMapper) {
        analysisInfoMapper.updateUpdateInfo(new Date(), userInfo.getUserName(),altaId);
    }

    /**
     * 更新数据
     *
     * @param userInfo
     * @param entity
     * @return
     */
    public Result<String> updateAnalysisInfo(HeaderUserInfo userInfo, AltaAnalysisInfo entity) {
        if (StringUtils.isBlank(entity.getAltaId()) || StringUtils.isBlank(entity.getAltaId())) {
            return Result.error(AuthorityProgramExceptionEnum.getLanguageMessageByKey(AuthorityProgramExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        analysisInfoMapper.updateById(entity);
        updateUpdateInfo(userInfo, entity.getAltaId(), analysisInfoMapper);
        return Result.ok();
    }

    /**
     * 逻辑删除数据
     *
     * @param userInfo
     * @param altaId
     * @return
     * @throws Exception
     */
    public Result<String> deleteAnalysisInfo(HeaderUserInfo userInfo, String altaId) {
        AltaAnalysisInfo analysis = analysisInfoMapper.selectById(altaId);
        if (Objects.equals(userInfo.getUserName(), analysis.getCreateBy())) {
            analysis.setDelFlag(AltaConstants.DEL_FLAG);
            analysisInfoMapper.updateById(analysis);
        }
        return Result.ok();
    }
}

