// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.alta.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.nationrel.modules.alta.mapper.*;
import com.nationrel.modules.alta.constant.AltaConstants;
import com.nationrel.modules.alta.constant.AltaDistributionTypeEnum;
import com.nationrel.modules.alta.constant.AltaLifeStressModelEnum;
import com.nationrel.modules.alta.dto.AltaAnalysisDataSaveDto;
import com.nationrel.modules.alta.entity.*;
import com.nationrel.modules.alta.general.common.ServerResponse;
import com.nationrel.modules.alta.general.domain.auth.HeaderUserInfo;
import com.nationrel.modules.alta.general.domain.enums.common.AuthorityProgramExceptionEnum;
import com.nationrel.modules.alta.general.util.common.BasicDataUtil;
import com.nationrel.modules.alta.util.ExportExcelUtil;
import com.nationrel.modules.alta.util.ImportExcelUtil;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @ClassName AltaAnalysisService
 * @Description 加速数据分析数据的操作
 * @Author hcs
 * @CreateDate 2021-04-26
 * @Version V1.0
 */
@Slf4j
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AltaAnalysisService {

    private final AltaAnalysisInfoMapper analysisInfoMapper;
    private final AltaCalcResultMapper calcResultMapper;
    private final AltaAnalysisMapper analysisMapper;
    private final AltaSettingMapper settingMapper;
    private final AltaColumnMapper columnMapper;
    private final AltaDataMapper dataMapper;
    private final AltaPlotMapper plotMapper;
    private final AltaColumnAndDataService columnAndDataService;

    /**
     * 分页查询
     *
     * @param userInfo
     * @param altaId
     * @return
     */
    public ServerResponse<List<AltaAnalysis>> listByPage(HeaderUserInfo userInfo, String altaId) {
        // 创建基础查询条件
        QueryWrapper<AltaAnalysis> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(AltaAnalysis::getCreateBy, userInfo.getUserName());
        queryWrapper.lambda().eq(AltaAnalysis::getDelFlag, AltaConstants.DEFAULT_DEL_FLAG);
        queryWrapper.lambda().eq(AltaAnalysis::getAltaId, altaId);
        queryWrapper.lambda().orderByDesc(AltaAnalysis::getCreateTime);
        return ServerResponse.createBySuccess(analysisMapper.selectList(queryWrapper));
    }

    /**
     * 添加分析数据信息
     *
     * @param userInfo
     * @param entity
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> addAnalysis(HeaderUserInfo userInfo, AltaAnalysis entity) {
        String analysisId = BasicDataUtil.uuidGenerator();
        entity.setAnalysisId(analysisId);
        entity.setCreateTime(new Date());
        entity.setCreateBy(userInfo.getUserName());
        entity.setUpdateTime(new Date());
        entity.setUpdateBy(userInfo.getUserName());
        entity.setDelFlag(AltaConstants.DEFAULT_DEL_FLAG);
        analysisMapper.insert(entity);

        List<AltaColumn> altaColumns = AltaColumn.initializeColumn(analysisId, userInfo.getLanguageKey());
        for (AltaColumn altaColumn : altaColumns) {
            altaColumn.setAnalysisId(analysisId);
            columnMapper.insert(altaColumn);
        }

        // ==================
        // 1.修改寿命数据分析数据表信息
        // ==================
        AltaSetting altaSetting = new AltaSetting();
        altaSetting.setDistributionType(AltaDistributionTypeEnum.Weibull.getIndex());
        altaSetting.setLifeStressModel(AltaLifeStressModelEnum.Arrhenius.getIndex());
        altaSetting.setAnalysisId(analysisId);
        // 包含删失数据(右删失数据)
        altaSetting.setRightFlag(AltaConstants.FLAG_NO_CHECKED);
        // 包含删失数据(右删失数据)
        altaSetting.setLeftFlag(AltaConstants.FLAG_NO_CHECKED);
        // 分组数据
        altaSetting.setGroupingFlag(AltaConstants.FLAG_NO_CHECKED);
        altaSetting.setCreateTime(new Date());
        altaSetting.setCreateBy(userInfo.getUserName());
        // 寿命数据分析数据表修改
        settingMapper.insert(altaSetting);
        AltaAnalysisInfoService.updateUpdateInfo(userInfo, entity.getAltaId(), analysisInfoMapper);
        return ServerResponse.createBySuccess(analysisId);
    }

    /**
     * 更新数据
     *
     * @param userInfo
     * @param entity
     * @return
     */
    public ServerResponse<String> updateAnalysis(HeaderUserInfo userInfo, AltaAnalysis entity) {
        if (StringUtils.isBlank(entity.getAnalysisId()) || (StringUtils.isBlank(entity.getAnalysisName()) && StringUtils.isBlank(entity.getEndTimeName()))) {
            return ServerResponse.createByErrorMessage(AuthorityProgramExceptionEnum
                    .getLanguageMessageByKey(AuthorityProgramExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        entity.setUpdateTime(new Date());
        entity.setUpdateBy(userInfo.getUserName());
        if (StringUtils.isNotBlank(entity.getEndTimeName())) {
            analysisMapper.updateEndTimeName(entity.getAnalysisId(), entity.getEndTimeName());
        } else {
            analysisMapper.updateById(entity);
        }
        AltaAnalysisInfoService.updateUpdateInfo(userInfo, entity.getAltaId(), analysisInfoMapper);
        return ServerResponse.createBySuccess();
    }

    /**
     * 更新分析名称
     *
     * @param userInfo
     * @param entity
     * @return
     */
    public ServerResponse<String> updateAnalysisName(HeaderUserInfo userInfo, AltaAnalysis entity) {
        if (StringUtils.isBlank(entity.getAnalysisId()) || StringUtils.isBlank(entity.getAnalysisName())) {
            return ServerResponse.createByErrorMessage(AuthorityProgramExceptionEnum
                    .getLanguageMessageByKey(AuthorityProgramExceptionEnum.所传数据不合法, userInfo.getLanguageKey()));
        }
        AltaAnalysis analysis = analysisMapper.selectById(entity.getAnalysisId());
        analysis.setAnalysisName(entity.getAnalysisName());
        analysis.setUpdateTime(new Date());
        analysis.setUpdateBy(userInfo.getUserName());
        analysisMapper.updateById(analysis);
        AltaAnalysisInfoService.updateUpdateInfo(userInfo, analysis.getAltaId(), analysisInfoMapper);
        return ServerResponse.createBySuccess();
    }

    /**
     * 删除数据
     *
     * @param analysisId
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> deleteAnalysis(String analysisId) {
        analysisMapper.deleteById(analysisId);
        dataMapper.delete(new QueryWrapper<AltaData>().lambda().eq(AltaData::getAnalysisId, analysisId));
        plotMapper.delete(new QueryWrapper<AltaPlot>().lambda().eq(AltaPlot::getAnalysisId, analysisId));
        columnMapper.delete(new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId, analysisId));
        settingMapper.delete(new QueryWrapper<AltaSetting>().lambda().eq(AltaSetting::getAnalysisId, analysisId));
        calcResultMapper
                .delete(new QueryWrapper<AltaCalcResult>().lambda().eq(AltaCalcResult::getAnalysisId, analysisId));
        return ServerResponse.createBySuccess();
    }

    /**
     * 复制数据
     *
     * @param analysisId
     * @return
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class)
    public ServerResponse<String> copyAnalysis(String analysisId) {
        String copyAnalysisId = BasicDataUtil.uuidGenerator();
        // 复制分析主信息
        AltaAnalysis analysis = analysisMapper.selectById(analysisId);
        analysis.setAnalysisId(copyAnalysisId);
        analysisMapper.insert(analysis);
        // 复制分析设置数据
        AltaSetting setting = settingMapper.selectById(analysisId);
        setting.setAnalysisId(copyAnalysisId);
        settingMapper.insert(setting);
        // 复制分析栏目信息
        List<AltaColumn> columns = columnMapper
                .selectList(new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId, analysisId));
        columns.forEach(column -> column.setAnalysisId(copyAnalysisId));
        columnMapper.insertBatch(columns);
        // 复制分析详细数据
        List<AltaData> datas = dataMapper
                .selectList(new QueryWrapper<AltaData>().lambda().eq(AltaData::getAnalysisId, analysisId));
        datas.forEach(data -> data.setAnalysisId(copyAnalysisId));
        dataMapper.insertBatch(datas);
        // 复制分析计算数据
        List<AltaCalcResult> calcResults = calcResultMapper
                .selectList(new QueryWrapper<AltaCalcResult>().lambda().eq(AltaCalcResult::getAnalysisId, analysisId));
        calcResults.forEach(calcResult -> {
            calcResult.setAnalysisId(copyAnalysisId);
            calcResultMapper.insert(calcResult);
        });
        // 复制分析图形数据
        List<AltaPlot> plots = plotMapper
                .selectList(new QueryWrapper<AltaPlot>().lambda().eq(AltaPlot::getAnalysisId, analysisId));
        plots.forEach(plot -> {
            plot.setAnalysisId(copyAnalysisId);
            plotMapper.insert(plot);
        });
        return ServerResponse.createBySuccess();
    }

    /**
     * 导入模板下载
     *
     * @param response
     * @return void
     * @throws IOException
     */
    public void importTemplate(HttpServletResponse response, String analysisId) {
        // 取得：Excel标题Map
        LinkedHashMap<String, String> excelTitleMap = this.buildExcelTitleMap(analysisId);
        // 导出：Excel
        ExportExcelUtil.export("加速寿命分析", new ArrayList<>(100), excelTitleMap, response);
    }

    /**
     * 数据导入
     *
     * @param userInfo   用户信息
     * @param file       文件
     * @param altaId     项目ID
     * @param analysisId 分析ID
     */
    @Transactional(rollbackFor = Exception.class)
    public void importData(HeaderUserInfo userInfo, MultipartFile file, String altaId, String analysisId) throws Exception {

        // 取得：Excel数据。
        List<List<Object>> excelList = ImportExcelUtil.getListByExcel(file.getInputStream(), file.getOriginalFilename(), userInfo.getLanguageKey());
        if (CollectionUtils.isEmpty(excelList)) {
            return;
        }
        // 组装数据。
        AltaAnalysisDataSaveDto entity = AltaAnalysisDataSaveDto.builder()
                .altaId(altaId)
                .analysisId(analysisId)
                .analysisDatas(this.excelToDatas(excelList))
                .startDataNo(1)
                .endDataNo(excelList.size() - 1)
                .hotDataChange(2)
                .groupingFlag(AltaConstants.FLAG_CHECKED)
                .leftFlag(AltaConstants.FLAG_CHECKED)
                .rightFlag(AltaConstants.FLAG_CHECKED)
                .build();
        // 1.保存数据（校验+保存）。
        columnAndDataService.updateData(userInfo, entity);

        // 2.修改：分析应力设置表。
        int otherTitleSize = excelList.get(0).size() - 4;
        this.updateAltaColumn(otherTitleSize, analysisId);
    }

    /**
     * 集合转JSON字符串
     *
     * @param excelList
     * @return
     */
    private List<String[]> excelToDatas(List<List<Object>> excelList) {
        List<String[]> altaData = new ArrayList<>();
        for (int i = 1; i < excelList.size(); i++) {
            List<String> objects = (List<String>) (List) excelList.get(i);
            altaData.add(new String[]{String.valueOf(i), this.getStr(objects, 0), this.getStr(objects, 1), this.getStr(objects, 2), this.getStr(objects, 3), this.getStr(objects, 4), this.getStr(objects, 5), this.getStr(objects, 6), this.getStr(objects, 7), this.getStr(objects, 8), this.getStr(objects, 9), this.getStr(objects, 10), this.getStr(objects, 11)});
        }
        return altaData;
    }

    /**
     * 取得：字符串
     *
     * @param objects 集合
     * @param index   索引
     * @return
     */
    private String getStr(List<String> objects, int index) {
        if (index < objects.size()) {
            return objects.get(index);
        }
        return null;
    }

    /**
     * 修改：分析应力设置表
     *
     * @param otherTitleSize 其他标题Size
     * @param analysisId     分析ID
     */
    private void updateAltaColumn(int otherTitleSize, String analysisId) {
        if (otherTitleSize <= 0) {
            return;
        }

        List<AltaColumn> altaColumnList = columnMapper.selectList(new QueryWrapper<AltaColumn>().lambda()
                .eq(AltaColumn::getAnalysisId, analysisId));
        for (AltaColumn altaColumn : altaColumnList) {
            if (otherTitleSize == 0) {
                break;
            }
            altaColumn.setSelectStatus(Integer.valueOf(AltaConstants.FLAG_CHECKED));
            otherTitleSize--;
        }
        // 批量修改：应力项配置
        columnMapper.batchUpdate(altaColumnList);
    }

    /**
     * 数据导出
     *
     * @param response
     * @param analysisId 数据ID
     * @throws IOException
     */
    public void exportData(HttpServletResponse response, String analysisId) {

        // 获取导出分析数据
        List<AltaData> altaDataList = dataMapper.selectList(new QueryWrapper<AltaData>().lambda()
                .eq(AltaData::getAnalysisId, analysisId)).stream().map(item -> {
            item.setFailureSuspension(AltaConstants.STATE_S.equals(item.getFailureSuspension()) || AltaConstants.STATE_SUSPENSION.equals(item.getFailureSuspension()) ? AltaConstants.STATE_SUSPENSION : AltaConstants.STATE_FUFPENSION);
            return item;
        }).collect(Collectors.toList());

        // 定义：sheet名称
        String sheetName = "加速寿命分析";
        // 取得：Excel标题Map
        LinkedHashMap<String, String> fieldMap = this.buildExcelTitleMap(analysisId);
        // 导出：Excel
        ExportExcelUtil.export(sheetName, altaDataList, fieldMap, response);
    }

    /**
     * 取得：Excel标题Map
     *
     * @param analysisId 分析ID
     * @return：Map
     */
    private LinkedHashMap<String, String> buildExcelTitleMap(String analysisId) {
        List<AltaColumn> columns = columnMapper.selectList(new QueryWrapper<AltaColumn>().lambda().eq(AltaColumn::getAnalysisId, analysisId).eq(AltaColumn::getSelectStatus, AltaConstants.FLAG_CHECKED));
        LinkedHashMap<String, String> excelTitleMap = new LinkedHashMap<>();
        excelTitleMap.put("quantity", "数量");
        excelTitleMap.put("lastInspectTime", "最近检查时间");
        excelTitleMap.put("failureSuspension", "F或S状态");
        excelTitleMap.put("failureEndTime", "结束时间");
        if (CollectionUtils.isEmpty(columns)) {
            return excelTitleMap;
        }
        columns.forEach(column -> excelTitleMap.put(column.getColumnId(), column.getName() + "(" + column.getUnit() + ")"));
        return excelTitleMap;
    }


    /**
     * 拼接：Excel标题
     * 格式：名称（单位）
     *
     * @return String
     */
    private String joinExcelTitle(String name, String unit) {
        return name + AltaConstants.LEFT_PARENTHESES_CN
                + unit + AltaConstants.RIGHT_PARENTHESES_CN;
    }

}
